//------------------------------------------------------------------------------------------------

//
// File: racetimes.cpp
// Author: ECE 275 Spring 2016
// Date: March 03, 2016
//
// Description:
//

//------------------------------------------------------------------------------------------------

#include <iostream>
#include <vector>
#include <fstream>
#include <string>
#include <sstream>
#include <climits>
#include "racetimes.h"
#include "timeddist.h"

//------------------------------------------------------------------------------------------------

using namespace std;

//------------------------------------------------------------------------------------------------

// Calculates the speed in MPH, averaged over the total time and distance traveled.
// The algorithm is: avgMpg = distance / time

float AvgSpeedMPH(TimedDist times[], int numTimes) {
   float totaldist = 0, totaltime = 0;
   float avgSpeed = 0;
   int i = 0;
   
   for (i = 0; i < numTimes; i++) {
      totaldist = totaldist + times[i].GetDistanceMiles();
      totaltime = totaltime + times[i].GetTimeSeconds() / 3600.0;
   }
   
   avgSpeed = totaldist / totaltime;
   return avgSpeed;
}

//------------------------------------------------------------------------------------------------

// Calculate the total distance traveled in miles
// The algorithm is
//   Loop over all the times
//     Update the total miles traveled
//   Return that value at the end.

float TotalDistMiles(TimedDist times[], int numTimes) {
   int i = 0;
   float sum_total = 0.0;
   
   for (i = 0; i < numTimes; i++) {
      sum_total += times[i].GetDistanceMiles();
   }
   
   return sum_total;
}

//------------------------------------------------------------------------------------------------

// Look at the times passed in, and calculate the number of seconds that have elapsed.
// The algorithm is:
//    Loop over all the times
//       Take the time in each slot (in minutes) and convert it to seconds
//       Update the total seconds
//    Return that value at the end.

float TotalTimeSec(TimedDist times[], int numTimes) {
   int i;
   float totalTime = 0;
   
   for (i = 0; i < numTimes; ++i) {
      totalTime += times[i].GetTimeSeconds();
   }
   
   
   return totalTime;
}

//------------------------------------------------------------------------------------------------

// Read race times from user using standard IO. Allows the user to enter an optional note
// at the end of the line.

void GetUserTimes(TimedDist times[], int numTimes) {
   int i = 0;
   char colon;
   float seconds = 0.0;
   float miles = 0.0;
   int minutes = 0;
   string userNote;
   string inputLine;
   string note;
   
   cout << "Number of raceTimes: " << numTimes << endl;
   
   for (i = 0; i < numTimes; ++i) {
      
      cout << "Enter timed distance (MMM:SS.SS MILES NOTE): ";
      getline(cin, inputLine);
      
      if (!cin.fail()) {
         istringstream lineStream(inputLine);
         lineStream >> minutes >> colon >> seconds >> miles;
         
         // fail() returns true if there is a logical error reading from the stream
         // In this situation, using good() will not work becuase when the end of the stream
         // (i.e. the string with which we initalized the stream), the iostream will
         // no longer be good. In other words, when the end of the stream is reached, the stream
         // is no longer good.
         if (lineStream.fail() || colon != ':') {
            continue;
         }
         
         // Attemp to read the rest of the line as the note.
         getline(lineStream, note);
         
         // Remove any leading spaces
         while (note.length() > 0 && note.at(0) == ' ') {
            note.erase(note.begin());
         }
         
         // If the note is empty, then we should not pass a note to the TimedDist constructor
         if (note.length() == 0) {
            times[i] = TimedDist(minutes, seconds, miles);
         }
         else {
            // Pass the note to the TimedDist constructor. Note that c_str() returns
            // a pointer to the C string (i.e., char *) for a string variable.
            // The returned pointer in returned as a "const char *"
            times[i] = TimedDist(minutes, seconds, miles, note.c_str());
         }
      }
      else {
         cin.clear();
      }
   }
}

//------------------------------------------------------------------------------------------------

// Read race times from file. This function does not support reading in a note from the file.

bool GetUserTimesFromFile(TimedDist times[], int numTimes, char* inputFileName) {
   
   return false;
}

//------------------------------------------------------------------------------------------------

// Prints all timed distance for race

void Print(TimedDist times[], int numTimes)
{
   unsigned int i = 0;
   
   cout << endl;
   for (i = 0; i < numTimes; ++i) {
      times[i].Print();
   }
}

//------------------------------------------------------------------------------------------------

